#!/usr/bin/perl -w

# Description: MTAmazon is a plugin designed for the Movable Type Publishing
#              Platform. It enables users to query and display items from
#              Amazon's product catalog within their published blog using
#              a few simple template tags.
#
# The copyright of this software is jointly owned between the following
#   parties:
# Copyright 2006 Byrne Reese
#
# $Id: $

package MT::Plugin::MTAmazon3;

use strict;
use MT;

use vars qw( $VERSION $plugin %cfg $DEBUG );
$VERSION = '3.09';
$DEBUG = 0;

eval {
    $plugin = new MT::Plugin({
	name            => 'MTAmazon',
        version         => $VERSION,
	author_name     => "Byrne Reese",
	author_link     => "http://www.majordojo.com/",
	plugin_link     => "http://www.majordojo.com/projects/MTAmazon/",
	description     => "MTAmazon is a Movable Type plugin that uses your Amazon Associate membership to retrieve products from Amazon. It allows flexible searching and display of products from any of Amazon's product categories.",
	doc_link        => "http://www.majordojo.com/projects/MTAmazon/docs.php",
	system_config_template => \&sysconf_template,
	blog_config_template => \&blogconf_template,
        object_classes => [ 'MTAmazon3::Item' ],
        schema_version => 1,
	settings        => new MT::PluginSettings([
                 ['accesskey',   { Default => '', 
                                   Scope => 'blog' }],
                 ['associateid', { Default => 'majordojo-20', 
                                   Scope => 'blog' }],
		 ['locale',      { Default => 'us',
			           Scope => 'blog' }],
		 ['delay',       { Default => 0,
			           Scope => 'blog' }],

		 ['accesskey',   { Default => '' }],
                 ['associateid', { Default => 'majordojo-20' }], 
		 ['allow_aid',   { Default => 1 }],
		 ['allow_sid',   { Default => 1 }],
        ]),
    });
    MT->add_plugin($plugin);
};

require MT::Template::Context;
MT::Template::Context->add_tag(AmazonTitle => sub { 
    require MTAmazon3::Plugin; &MTAmazon3::Plugin::AmazonTitle; 
});
MT::Template::Context->add_tag(AmazonDetailPageURL => sub { 
    require MTAmazon3::Plugin; &MTAmazon3::Plugin::AmazonDetailPageURL; 
});
MT::Template::Context->add_tag(AmazonASIN => sub { 
    require MTAmazon3::Plugin; &MTAmazon3::Plugin::AmazonASIN; 
});
MT::Template::Context->add_tag(AmazonProductGroup => sub { 
    require MTAmazon3::Plugin; &MTAmazon3::Plugin::AmazonProductGroup; 
});
MT::Template::Context->add_tag(AmazonItemAttributes => sub { 
    require MTAmazon3::Plugin; &MTAmazon3::Plugin::AmazonItemAttributes; 
});
MT::Template::Context->add_tag(AmazonImageTag => sub { 
    require MTAmazon3::Plugin; &MTAmazon3::Plugin::AmazonImage; 
});
MT::Template::Context->add_tag(AmazonCustomImageURL => sub { 
    require MTAmazon3::Plugin; &MTAmazon3::Plugin::AmazonCustomImage; 
});
MT::Template::Context->add_tag(AmazonPrice => sub { 
    require MTAmazon3::Plugin; &MTAmazon3::Plugin::AmazonPrice; 
});
MT::Template::Context->add_tag(AmazonField => sub { 
    require MTAmazon3::Plugin; &MTAmazon3::Plugin::AmazonField; 
});

MT::Template::Context->add_tag(AmazonListId => sub { 
    require MTAmazon3::Plugin; &MTAmazon3::Plugin::AmazonListId; 
});
MT::Template::Context->add_tag(AmazonListURL => sub { 
    require MTAmazon3::Plugin; &MTAmazon3::Plugin::AmazonListURL; 
});
MT::Template::Context->add_tag(AmazonListName => sub { 
    require MTAmazon3::Plugin; &MTAmazon3::Plugin::AmazonListName; 
});
MT::Template::Context->add_tag(AmazonListType => sub { 
    require MTAmazon3::Plugin; &MTAmazon3::Plugin::AmazonListType; 
});

MT::Template::Context->add_container_tag(AmazonItemLookup => sub { 
    require MTAmazon3::Plugin; &MTAmazon3::Plugin::ItemLookup; 
});
MT::Template::Context->add_container_tag(AmazonItemSearch => sub { 
    require MTAmazon3::Plugin; &MTAmazon3::Plugin::ItemSearch; 
});
MT::Template::Context->add_container_tag(AmazonSimilarityLookup => sub { 
    require MTAmazon3::Plugin; &MTAmazon3::Plugin::SimilarityLookup; 
});
MT::Template::Context->add_container_tag(AmazonListSearch => sub { 
    require MTAmazon3::Plugin; &MTAmazon3::Plugin::ListSearch; 
});
MT::Template::Context->add_container_tag(AmazonListLookup => sub { 
    require MTAmazon3::Plugin; &MTAmazon3::Plugin::ListLookup; 
});

MT::Template::Context->add_conditional_tag(AmazonIfItemAttribute => sub { 
    require MTAmazon3::Plugin; &MTAmazon3::Plugin::AmazonIfItemAttribute; 
});

sub blogconf_template {
    my ($plugin,$param,$scope) = @_;
    require MTAmazon3::Util;
    my $cfg = MTAmazon3::Util::readconfig(MT::App->instance->blog->id,
					  { ignore_errors => 1} );
    my $tmpl = "";
    if ($cfg->{allow_sid}) {
	$tmpl .= <<'EOT';
    <div class="setting">
      <div class="label">
        <label for="accesskey">Access Key:</label>
      </div>
      <div class="field">
        <p><input type="text" size="30" name="accesskey" value="<TMPL_VAR NAME=ACCESSKEY>" /><br />
        <a target="_new" href="http://www.amazon.com/gp/aws/registration/registration-form.html">Register for an Access Key</a> | 
        <a target="_new" href="">What is an Access Key?</a></p>
      </div>
    </div>
EOT
    }
    if ($cfg->{allow_aid}) {
        $tmpl .= <<'EOT';
    <div class="setting">
      <div class="label">
        <label for="associateid">Associates Id:</label>
      </div>
      <div class="field">
        <p><input type="text" size="30" name="associateid" value="<TMPL_VAR NAME=ASSOCIATEID>" /></p>
      </div>
    </div>
EOT
    }
    $tmpl .= <<'EOT';
    <div class="setting">
      <div class="label">
        <label for="locale">Locale:</label>
      </div>
      <div class="field">
        <select name="locale">
EOT
    my %locales = ( 'us' => "United States",
		    'uk' => "United Kingdom",
		    'de' => "Germany",
		    'jp' => "Japan",
		    'fr' => "France",
		    'ca' => "Canada",
		    );
    foreach my $l (sort keys %locales) {
	$tmpl .= "<option value=\"$l\"".($l eq $param->{'locale'} ? " selected" : "").">".$locales{$l}."</option>\n";
    }
     $tmpl .= <<EOT;
        </select>
      </div>
    </div> 
    <div class="setting">
      <div class="label">
      </div>
      <div class="field">
        <p><strong>Insert a 1 second delay between Amazon inquires?<sup>*</sup></strong></p>
        <p>
EOT
    $tmpl .= '<input type="radio" id="delay_yes" name="delay" value="1" '.($cfg->{delay} ? "checked " : "").'/> <label for="delay_yes">Yes</label>';
    $tmpl .= '&nbsp;&nbsp;';
    $tmpl .= '<input type="radio" id="delay_no" name="delay" value="0" '.(!$cfg->{delay} ? "checked " : "").'/> <label for="delay_no">No</label>';
	  $tmpl .= <<EOT;
        </p>
        <p style="font-size: 80%"><sup>*</sup> Amazon will often rate limit the number of requests a given subscriber ID is allowed to make if that ID makes more than one request per second. Inserting a delay will alleviate this problem. Only set this to 'Yes' if you are receiving error messages from Amazon, or your rebuilds will get very slow.</p>
      </div>
    </div>
EOT
}

sub sysconf_template {
    my $tmpl = <<'EOT';
    <div class="setting">
      <div class="label">
        <label for="accesskey">Access Key:</label>
      </div>
      <div class="field">
        <p><input type="text" size="30" name="accesskey" value="<TMPL_VAR NAME=ACCESSKEY>" /><br />
        <a target="_new" href="http://www.amazon.com/gp/aws/registration/registration-form.html">Register for an Access Key</a> | 
        <a target="_new" href="">What is an Access Key?</a></p>
      </div>
      <div class="label"></div>
      <div class="field">
        <p><input type="checkbox" name="allow_sid" value="1" <TMPL_IF NAME=ALLOW_SID>checked </TMPL_IF>/> Permit blog owners to use their own Amazon Access Key.</p>
      </div>
    </div>
    <div class="setting">
      <div class="label">
        <label for="devtoken">Associates Id:</label>
      </div>
      <div class="field">
        <p><input type="text" size="30" name="associateid" value="<TMPL_VAR NAME=ASSOCIATEID>" /></p>
      </div>
      <div class="label"></div>
      <div class="field">
        <p><input type="checkbox" name="allow_aid" value="1" <TMPL_IF NAME=ALLOW_AID>checked </TMPL_IF>/> Permit blog owners to use their own Amazon Associates ID.</p>
      </div>
    </div>
    <div class="setting">
      <div class="label"></div>
      <div class="field">
        <p>
	<input type="button" value="Clear MTAmazon Cache" onclick="window.open('plugins/MediaManager/mtamazon.cgi?__mode=clear_cache', 'clear', 'width=400,height=450,resizable=yes')" />
        </p>
      </div>
    </div> 
EOT
}

1;
