#!/usr/bin/perl -w

# Copyright 2005 Byrne Reese
#
# $Id: $

use Cwd qw/cwd chdir abs_path/;
use File::Basename qw/dirname/;
use File::Spec;

package MT::Plugin::MediaManager;

use strict;
use MT;
use MT::Template::Context;
use MT::Util qw( mark_odd_rows format_ts);

use XML::Simple;

use MediaManager::Plugin;

use vars qw( $VERSION );
$VERSION = '1.1';

my $plugin;
eval {
    $plugin = MediaManager::Plugin->new({
      name        => 'Media Manager',
      description => "Maintain and review a list of items pulled from Amazon's product catalog.",
      version     => $VERSION,
      author_name => "Byrne Reese",
      author_link => "http://www.majordojo.com/",
      plugin_link => "http://www.majordojo.com/projects/MediaManager/",
      doc_link    => "http://www.majordojo.com/projects/MediaManager/docs.php",
      object_classes => [ 'MediaManager::Entry','MediaManager::Review' ],
      schema_version => 2,
      upgrade_functions => {
	  'my_plugin_fix_field_a' => {
            version_limit => 2,
            code => sub { require MediaManager::Upgrade; &MediaManager::Upgrade::convert_status_to_tag; },
	    }
      }
    });
    MT->add_plugin($plugin);
};

MT->add_plugin_action('blog','mmanager.cgi?',"Edit My Media Queue");

MT::Entry->add_callback("post_load", 
			10, 
			$plugin, 
			\&MediaManager::Plugin::post_load);

MT::Entry->add_callback("pre_save", 
			10, 
			$plugin, 
			\&MediaManager::Plugin::pre_save);

MT::Entry->add_callback("post_save", 
			10, 
			$plugin, 
			\&MediaManager::Plugin::post_save);

MT->add_callback("AppTemplateSource.edit_entry",
		 10,
		 $plugin,
		 \&MediaManager::Plugin::transform);

MT->add_callback("AppTemplateSource.blog-left-nav",
		 10,
		 $plugin,
		 \&MediaManager::Plugin::left_nav);

# NOTE: any mention here of an `entry_id' refers to a regular MT blog
# entry that may be associated with a Media Manager entry (Media Manager
# entries themselves do not have entry_id's)

# Template Tag Definitions
# MediaManager::Template::ContextHandlers is only required once
MT::Template::Context->add_container_tag(MediaManagerItems => sub {
    require MediaManager::Template::ContextHandlers; 
    &MediaManager::Template::ContextHandlers::hdlr_mediamanager_entries; 
}); 
MT::Template::Context->add_container_tag(MediaManagerRandomItem => sub { 
    require MediaManager::Template::ContextHandlers;
    &MediaManager::Template::ContextHandlers::mm_random_item;
});

MT::Template::Context->add_tag(ItemCount          => 
	       \&MediaManager::Template::ContextHandlers::hdlr_itemcount);
MT::Template::Context->add_tag(ItemISBN           => 
	       \&MediaManager::Template::ContextHandlers::hdlr_itemisbn);
MT::Template::Context->add_tag(ItemASIN           => 
	       \&MediaManager::Template::ContextHandlers::hdlr_itemisbn);
MT::Template::Context->add_tag(ItemStatus         => 
	       \&MediaManager::Template::ContextHandlers::hdlr_itemstatus);
MT::Template::Context->add_tag(ItemCatalog        => 
	       \&MediaManager::Template::ContextHandlers::hdlr_itemcatalog);
MT::Template::Context->add_tag(ItemTitle          => 
	       \&MediaManager::Template::ContextHandlers::hdlr_itemtitle);
MT::Template::Context->add_tag(ItemIcon           => 
	       \&MediaManager::Template::ContextHandlers::hdlr_itemicon);
MT::Template::Context->add_tag(ItemFinishedDate   => 
	       \&MediaManager::Template::ContextHandlers::hdlr_itemfinishedon);
MT::Template::Context->add_tag(ItemCreatedDate    => 
	       \&MediaManager::Template::ContextHandlers::hdlr_itemcreatedon);
MT::Template::Context->add_tag(ItemModifiedDate   => 
	       \&MediaManager::Template::ContextHandlers::hdlr_itemmodifiedon);
MT::Template::Context->add_tag(ItemEntryDate      => 
	       \&MediaManager::Template::ContextHandlers::hdlr_itemcreatedon);
MT::Template::Context->add_tag(ItemEntryID        => 
	       \&MediaManager::Template::ContextHandlers::hdlr_itementryid);
MT::Template::Context->add_tag(ItemEntryLink      => 
	       \&MediaManager::Template::ContextHandlers::hdlr_itementrylink);
MT::Template::Context->add_tag(ItemEntryPermalink => 
	       \&MediaManager::Template::ContextHandlers::hdlr_itementrylink);

MT::Template::Context->add_conditional_tag(ItemIfBlogEntry    => 
	       \&MediaManager::Template::ContextHandlers::hdlr_itemifentry);
MT::Template::Context->add_conditional_tag(ItemIfFinishedDate => 
	       \&MediaManager::Template::ContextHandlers::hdlr_itemiffinishedon);

MT::Template::Context->add_tag(EntryIfReview      => 
	       \&MediaManager::Template::ContextHandlers::hdlr_itemifentry);
MT::Template::Context->add_tag(ReviewRatingNumber => 
	       \&MediaManager::Template::ContextHandlers::mm_review_rating_number);
MT::Template::Context->add_tag(ReviewRatingName   => 
	       \&MediaManager::Template::ContextHandlers::mm_review_rating_name);

for my $rating (0..5) {
    MT::Template::Context->add_conditional_tag( "ReviewRatingIs$rating" => 
		\&MediaManager::Template::ContextHandlers::mm_review_rating_is );
}

# End Template Tag Definitions

1;

__END__

