# MTAmazon Movable Type Plugin
#
# $Id: $
#
# Copyright (C) 2006 Byrne Reese
#

package MTAmazon3::Plugin;

use strict;
use Exporter;
use MT::Plugin;
use vars qw(@EXPORT_OK @EXPORT);
use MTAmazon3::Util qw(debug trim readconfig find_cached_item cache_item CallAmazon handle_expressions);
@MTAmazon::Plugin3::ISA = qw( MT::Plugin Exporter );
@EXPORT = qw(ItemLookupHelper ItemSearch ListLookup ListSearch);

sub repair_class {
    require MT::App;
    my $app = MT::App->instance;

    my ($class) = @_;
    require MT::Upgrade;
    my $driver = MT::Object->driver;
    my $diff   = MT::Upgrade->class_diff($class);
    if ($diff) {
        my @stmt;
        if ($diff->{fix}) {
            @stmt = $driver->fix_class($class);
        } else {
            if ($diff->{add}) {
                push @stmt, $driver->add_column($class, $_->{name})
                    foreach @{$diff->{add}};
            }
            if ($diff->{alter}) {
                push @stmt, $driver->alter_column($class, $_->{name})
                    foreach @{$diff->{alter}};
            }
            if ($diff->{drop}) {
                push @stmt, $driver->drop_column($class, $_->{name})
                    foreach @{$diff->{drop}};
            }
        }
        if (@stmt) {
            $driver->sql(\@stmt) or return $app->error($driver->errstr);
        }
    }
}

sub install_check {
    my ($ctx) = @_;
    return if ($ctx->stash('mtamazon-installed'));
    require MT::Object;
    require MTAmazon3::Item;
    my $driver = MT::Object->driver;
    if (!$driver || !$driver->table_exists('MTAmazon3::Item')) {
	debug("Installing MTAmazon3...");
	my $res = return repair_class('MTAmazon3::Item');
	debug("Installation of MTAmazon3 complete.");
	$ctx->stash('mtamazon-installed',1) unless $res;
	return $res;
    }
}

sub ItemLookupHelper {
    my ($config,$args) = @_;
    my $content_tree;
    my $item;
    require XML::Simple;
    my $p = new XML::Simple;
    debug("Looking for cached item (".$args->{ItemId}.")...");
    if ($item = find_cached_item($args->{'ItemId'},$args->{'ResponseGroup'})) {
	debug("Found cached item (".$args->{ItemId}.").");
	$content_tree = $item->eval_data();
    } else {
	debug("Calling Amazon for item (".
			       $args->{ItemId}.
			       ").");
	my $content = CallAmazon("ItemLookup",$config,$args);
	eval { $content_tree = $p->XMLin($content); };
	if ($content_tree && $content_tree->{'Error'}) {
	    die "Amazon returned the following error: " .
		$content_tree->{Error}->{Message};
	} elsif ($@) {
	    die "Error reading response from Amazon. It is " .
		"possible that Amazon returned an HTTP Status " .
		"of 500 due to an intermittent problem on their " .
		"end. Here is the content of their response: " .
		$content;
	}
	debug("Caching item (".$args->{ItemId}.").");
	cache_item($args->{'ItemId'},
		   $args->{'ResponseGroup'},
		   $content_tree);
    }
    return $content_tree;
}

sub ItemLookup {
    my ($ctx, $args) = @_;
    if (my $err = install_check($ctx)) { return $err; }

    if (!defined($args->{'ResponseGroup'})) {
	$args->{'ResponseGroup'} = 'Small,Images,OfferSummary';
    }
    my $config = readconfig($ctx->stash('blog_id'));

    my $tokens = $ctx->stash('tokens');
    my $builder = $ctx->stash('builder');

    $args = handle_expressions($ctx, $args);
#    use Data::Dumper;
#    debug(Dumper($args));

    debug("Looking for cached item (".$args->{ItemId}.")...");
    my $content_tree;
    eval {
	$content_tree = ItemLookupHelper($config,$args);
    };
    if ($@) {
	return $ctx->error($@);
    }
#    $ctx->stash('AmazonXML', $p->XMLout($content_tree));
    my $details = $content_tree->{'Items'}->{'Item'};
    return '' unless defined $details; 
    my $prod = '';
    $ctx->stash('AmazonItem', $details);
    my $out = $builder->build($ctx, $tokens);
    return $ctx->error( $builder->errstr ) unless defined $out;
    $prod .= $out;
    return $prod;
}

sub ItemSearch {
    my ($ctx, $args) = @_;
    if (my $err = install_check($ctx)) { return $err; }

    if (!defined($args->{'ResponseGroup'})) {
	$args->{'ResponseGroup'} = 'Small,Images,OfferSummary';
    }
    my $config = readconfig($ctx->stash('blog_id'));

    my $tokens = $ctx->stash('tokens');
    my $builder = $ctx->stash('builder');
    $args = handle_expressions($ctx, $args);

    my $lastn = $args->{lastn} || 1;
    my $pages = int(($lastn/10) + .9999);
    my $items = [];
    debug("Fetching $pages pages from Amazon");
    for (my $page = 1; $page <= $pages; $page++) {
	my $content_tree;
	debug("Looking for cached item (".$args->{ItemId}.")...");
	my $item;
	require XML::Simple;
	my $p = new XML::Simple;
	$args->{'ItemPage'} = $page;
	debug("Conducting search for '".$args->{Keywords}."'");
	my $content = CallAmazon("ItemSearch",$config,$args);
	eval { $content_tree = $p->XMLin($content); };
	if ($content_tree && $content_tree->{'Error'}) {
	    return $ctx->error("Amazon returned the following error: ".
			       $content_tree->{Error}->{Message});
	} elsif ($@) {
	    return $ctx->error("Error reading response from Amazon. It is ".
			       "possible that Amazon returned an HTTP Status ".
			       "of 500 due to an intermittent problem on their ".
			       "end. Here is the content of their response: ".
			       $content);
	}

	$ctx->stash('AmazonXML', $p->XMLout($content_tree));

	if ($page == 1 && 
	    $content_tree->{'Items'}->{'TotalPages'} && 
            $content_tree->{'Items'}->{'TotalPages'} < $pages) {
	    $pages = $content_tree->{'Items'}->{'TotalPages'};
	}

	my $details;
	if ((ref $content_tree->{'Items'}->{'Item'} eq 'HASH')) {
	    $details = [ $content_tree->{'Items'}->{'Item'} ];
	} else {
	    $details = $content_tree->{'Items'}->{'Item'};
	}
	$details = [] if !$details;
	push @$items, @$details;
    }

    return '' unless defined @$items; 
    my $prod;
    my $count = 0;
    for my $i (@$items) {
        last if ++$count > $lastn;
        $ctx->stash('AmazonItem', $i);
        my $out = $builder->build($ctx, $tokens);
        return $ctx->error( $builder->errstr ) unless defined $out;
        $prod .= $out;
    }
    return $prod;
}

sub SimilarityLookup {
    my ($ctx, $args) = @_;
    if (my $err = install_check($ctx)) { return $err; }

    if (!defined($args->{'ResponseGroup'})) {
	$args->{'ResponseGroup'} = 'Small,Images,OfferSummary';
    }
    my $config = readconfig($ctx->stash('blog_id'));

    my $tokens = $ctx->stash('tokens');
    my $builder = $ctx->stash('builder');
    $args = handle_expressions($ctx, $args);

    my $lastn = $args->{lastn} || 1;
    my $pages = int(($lastn/10) + .9999);
    my $items = ();
    debug("Fetching $pages pages from Amazon");
    for (my $page = 1; $page <= $pages; $page++) {
	my $content_tree;
	debug("Looking for cached item (".$args->{ItemId}.")...");
	my $item;
	require XML::Simple;
	my $p = new XML::Simple;
	$args->{'ItemPage'} = $page;
	my $content = CallAmazon("SimilarityLookup",$config,$args);
	eval { $content_tree = $p->XMLin($content); };
	if ($content_tree && $content_tree->{'Error'}) {
	    return $ctx->error("Amazon returned the following error: ".
			       $content_tree->{Error}->{Message});
	} elsif ($@) {
	    return $ctx->error("Error reading response from Amazon. It is ".
			       "possible that Amazon returned an HTTP Status ".
			       "of 500 due to an intermittent problem on their ".
			       "end. Here is the content of their response: ".
			       $content);
	}

	$ctx->stash('AmazonXML', $p->XMLout($content_tree));

	if ($page == 1 && 
	    $content_tree->{'Items'}->{'TotalPages'} && 
            $content_tree->{'Items'}->{'TotalPages'} < $pages) {
	    $pages = $content_tree->{'Items'}->{'TotalPages'};
	}

	my $details;
	if ((ref $content_tree->{'Items'}->{'Item'} eq 'HASH')) {
	    $details = [ $content_tree->{'Items'}->{'Item'} ];
	} elsif ((ref $content_tree->{'Items'}->{'Item'} eq 'ARRAY')) {
	    $details = $content_tree->{'Items'}->{'Item'};
	} else {
	    return $ctx->error("Unknown response from Amazon - response was of type: " .
			       ref $content_tree->{'Items'}->{'Item'});
	}
	push @$items, @$details;
    }

    return '' unless defined @$items; 
    my $prod;
    my $count = 0;
    for my $i (@$items) {
        last if ++$count > $lastn;
        $ctx->stash('AmazonItem', $i);
        my $out = $builder->build($ctx, $tokens);
        return $ctx->error( $builder->errstr ) unless defined $out;
        $prod .= $out;
    }
    return $prod;
}

sub ListSearch {
    my ($ctx, $args) = @_;
    if (my $err = install_check($ctx)) { return $err; }

    if (!defined($args->{'ResponseGroup'})) {
	$args->{'ResponseGroup'} = 'Request,ListInfo';
    }
    if (!defined($args->{'ListType'})) {
	$args->{'ListType'} = 'WishList';
    }
    my $config = readconfig($ctx->stash('blog_id'));

    my $tokens = $ctx->stash('tokens');
    my $builder = $ctx->stash('builder');
    $args = handle_expressions($ctx, $args);

    my $lastn = $args->{lastn} || 1;
    my $pages = int(($lastn/10) + .9999);
    my $lists;
    debug("Fetching $pages pages from Amazon");
    for (my $page = 1; $page <= $pages; $page++) {
	my $content_tree;
	debug("Looking for cached item (".$args->{ListId}.")...");
	my $list;
	require XML::Simple;
	my $p = new XML::Simple;
	$args->{'ListPage'} = $page;
	my $content = CallAmazon("ListSearch",$config,$args);
	eval { $content_tree = $p->XMLin($content); };
	if ($content_tree && 
	    $content_tree->{'Lists'}->{'Request'}->{'Errors'}->{'Error'}) {
	    return $ctx->error("Amazon returned the following error: ".
	       $content_tree->{'Lists'}->{'Request'}->{'Errors'}->{'Error'}->{'Message'});
	} elsif ($@) {
	    return $ctx->error("Error reading response from Amazon. It is ".
			       "possible that Amazon returned an HTTP Status ".
			       "of 500 due to an intermittent problem on their ".
			       "end. Here is the content of their response: ".
			       $content);
	}

	$ctx->stash('AmazonXML', $p->XMLout($content_tree));

	if ($page == 1 && 
	    $content_tree->{'Lists'}->{'TotalPages'} && 
            $content_tree->{'Lists'}->{'TotalPages'} < $pages) {
	    $pages = $content_tree->{'Lists'}->{'TotalPages'};
	}

	my $details;
	if ((ref $content_tree->{'Lists'}->{'List'} eq 'HASH')) {
	    $details = [ $content_tree->{'Lists'}->{'List'} ];
	} else {
	    $details = $content_tree->{'Lists'}->{'List'};
	}
	push @$lists, @$details;
    }

    return '' unless defined @$lists; 
    my $prod;
    my $count = 0;
    for my $i (@$lists) {
        last if ++$count > $lastn;
        $ctx->stash('AmazonList', $i);
        my $out = $builder->build($ctx, $tokens);
        return $ctx->error( $builder->errstr ) unless defined $out;
        $prod .= $out;
    }
    return $prod;
}

sub ListLookup {
    my ($ctx, $args) = @_;
    if (my $err = install_check($ctx)) { return $err; }

    if (!defined($args->{'ResponseGroup'})) {
	$args->{'ResponseGroup'} = 'ItemAttributes,ListInfo,ListItems,Small,Images,OfferSummary';
    }
    if (!defined($args->{'ListType'})) {
	$args->{'ListType'} = 'WishList';
    }
    my $config = readconfig($ctx->stash('blog_id'));

    my $tokens = $ctx->stash('tokens');
    my $builder = $ctx->stash('builder');
    $args = handle_expressions($ctx, $args);

    my $lastn = $args->{lastn} || 1;
    my $pages = int(($lastn/10) + .9999);
    my $items;
    debug("Fetching $pages pages from Amazon");
    for (my $page = 1; $page <= $pages; $page++) {
	my $content_tree;
	debug("Looking for cached item (".$args->{ListId}.")...");
	my $item;
	require XML::Simple;
	my $p = new XML::Simple;
	$args->{'ItemPage'} = $page;
	my $content = CallAmazon("ListLookup",$config,$args);
	eval { $content_tree = $p->XMLin($content); };
	if ($content_tree && 
	    $content_tree->{'Lists'}->{'Request'}->{'Errors'}->{'Error'}) {
	    return $ctx->error("Amazon returned the following error: ".
	       $content_tree->{'Lists'}->{'Request'}->{'Errors'}->{'Error'}->{'Message'});
	} elsif ($@) {
	    return $ctx->error("Error reading response from Amazon. It is ".
			       "possible that Amazon returned an HTTP Status ".
			       "of 500 due to an intermittent problem on their ".
			       "end. Here is the content of their response: ".
			       $content);
	}

	$ctx->stash('AmazonXML', $p->XMLout($content_tree));
#	debug($ctx->stash('AmazonXML'));

	if ($page == 1 && 
	    $content_tree->{'Lists'}->{'TotalPages'} && 
            $content_tree->{'Lists'}->{'TotalPages'} < $pages) {
	    $pages = $content_tree->{'Lists'}->{'TotalPages'};
	}

	my $details;
	if ((ref $content_tree->{'Lists'}->{'List'}->{'ListItem'} eq 'HASH')) {
	    $details = [ $content_tree->{'Lists'}->{'List'}->{'ListItem'} ];
	} else {
	    $details = $content_tree->{'Lists'}->{'List'}->{'ListItem'};
	}
	push @$items, @$details;
    }

    return '' unless defined @$items; 
    my $prod;
    my $count = 0;
    for my $i (@$items) {
        last if ++$count > $lastn;
        $ctx->stash('AmazonItem', $i->{Item});
        my $out = $builder->build($ctx, $tokens);
        return $ctx->error( $builder->errstr ) unless defined $out;
        $prod .= $out;
    }
    return $prod;
}

sub AmazonListId {
    my $ctx = shift;
    defined(my $i = $ctx->stash('AmazonList'))
        or return '';
    $i->{ListId} || '';
}

sub AmazonListURL {
    my $ctx = shift;
    defined(my $i = $ctx->stash('AmazonList'))
        or return '';
    $i->{ListURL} || '';
}

sub AmazonListName {
    my $ctx = shift;
    defined(my $i = $ctx->stash('AmazonList'))
        or return '';
    $i->{ListName} || '';
}

sub AmazonListType {
    my $ctx = shift;
    defined(my $i = $ctx->stash('AmazonList'))
        or return '';
    $i->{ListType} || '';
}

sub AmazonASIN {
    my $ctx = shift;
    defined(my $i = $ctx->stash('AmazonItem'))
        or return '';
    $i->{ASIN} || '';
}

sub AmazonDetailPageURL {
    my $ctx = shift;
    defined(my $i = $ctx->stash('AmazonItem'))
        or return '';
    $i->{DetailPageURL} || '';
}

sub AmazonTitle {
    my $ctx = shift;
    defined(my $i = $ctx->stash('AmazonItem'))
        or return '';
    $i->{ItemAttributes}->{Title} || '';
}

sub AmazonProductGroup {
    my $ctx = shift;
    defined(my $i = $ctx->stash('AmazonItem'))
        or return '';
    $i->{ItemAttributes}->{ProductGroup} || '';
}

sub AmazonIfItemAttribute {
    my ($ctx, $args) = @_;
    defined(my $i = $ctx->stash('AmazonItem'))
        or return 0;
    my $name = $args->{'name'};
    my $elem = $i->{ItemAttributes}->{$name};
    return defined($elem);
}

sub AmazonItemAttributes {
    my ($ctx, $args) = @_;
    defined(my $i = $ctx->stash('AmazonItem'))
        or return '';
    my $name = $args->{'name'};
    my $elem = $i->{ItemAttributes}->{$name};
    if (!defined($elem)) {
	return '';
    } elsif ((ref $elem eq 'ARRAY')) {
	return join(", ",@$elem);
    } else {
	return $elem;
    }
}

sub AmazonPrice {
    my ($ctx, $args) = @_;
    defined(my $i = $ctx->stash('AmazonItem'))
        or return '';
    my $name = lc($args->{'type'}) || 'New';
    $name =~ s/^(.)(.*)/uc($1).lc($2)/e;
    return $i->{OfferSummary}->{'Lowest'.$name.'Price'}->{FormattedPrice};
}

sub AmazonImage {
    my ($ctx, $args) = @_;
    defined(my $i = $ctx->stash('AmazonItem'))
        or return '';
    my $size = $args->{'size'};
    $size =~ s/^(.)(.*)/uc($1).lc($2)/e;
    my $url = $i->{$size.'Image'}->{URL};
    my $h   = $i->{$size.'Image'}->{Height}->{content};
    my $w   = $i->{$size.'Image'}->{Width}->{content};
    return '<img src="'.$url.'" width="'.$w.'" height="'.$h.'" />';
}

sub AmazonCustomImage {
    my ($ctx,$args) = @_;
    defined(my $i = $ctx->stash('AmazonItem'))
        or return '';
    my $opts = "_";
    $opts .= "THUMBZZZ_" if $args->{'size'} eq "thumb";
    $opts .= "TZZZZZZZ_" if $args->{'size'} eq "small";
    $opts .= "SCMZZZZZZZ_" if $args->{'size'} eq "medium";
    $opts .= "SCLZZZZZZZ_" if $args->{'size'} eq "large";
    $opts .= "AA".$args->{'width'}."_" if $args->{'width'};
    if ($opts eq '_') { # user did not specify a size
	$opts .= "SCMZZZZZZZ_";
    }
    $opts .= "PU".$args->{'rotation'}."_" if $args->{'rotation'};
    $opts .= "BL".$args->{'blur'}."_" if $args->{'blur'};
    $opts .= "PB_" if $args->{'shadow'} eq 'left';
    $opts .= "PC_" if $args->{'shadow'} eq 'right';
    if ($args->{'percent'}) {
	$opts .= "PD".$args->{'percent'}."_" if $args->{'percentloc'} eq 'left';
	$opts .= "PE".$args->{'percent'}."_" if $args->{'percentloc'} eq 'right';
    }
    return 'http://images.amazon.com/images/P/'.$i->{ASIN}.'.01.'.$opts.'.jpg';
}

sub AmazonField { 
    my ($ctx, $args) = @_; 
    $args = handle_expressions($ctx, $args);
    my $name = $args->{name}; 
    return '' unless $name; 
    defined(my $i = $ctx->stash('AmazonItem')) 
     or return ''; 
    if ($name =~ m|/|) {
        my @path = split(/\//, $name);
        $name = pop @path;
        for my $node (@path) {
            $i = $i->{$node};
        }
    }
    $i->{$name} || ''; 
} 

1;

__END__

#<?xml version="1.0" encoding="UTF-8"?>
#<ItemLookupResponse xmlns="http://webservices.amazon.com/AWSECommerceService/2005-10-05">
#  <OperationRequest>
#    <HTTPHeaders>
#      <Header Name="UserAgent" Value="MTAmazon/3.0"></Header>
#    </HTTPHeaders>
#    <RequestId>0WACKY4EYNR9VMZAKZ7Q</RequestId>
#    <Arguments>
#      <Argument Name="Service" Value="AWSECommerceService"></Argument>
#      <Argument Name="AWSAccessKeyId" Value="1FNQS2WS91241WGETX82"></Argument>
#      <Argument Name="ItemId" Value="B00005JNOG"></Argument>
#      <Argument Name="Operation" Value="ItemLookup"></Argument>
#    </Arguments>
#    <RequestProcessingTime>0.00972390174865723</RequestProcessingTime>
#  </OperationRequest>
#  <Items>
#    <Request>
#      <IsValid>True</IsValid>
#      <ItemLookupRequest>
#        <ItemId>B00005JNOG</ItemId>
#      </ItemLookupRequest>
#    </Request>
#    <Item>
#      <ASIN>B00005JNOG</ASIN>
#      <DetailPageURL>...</DetailPageURL>
#      <SmallImage>
#        <URL>http://images.amazon.com/images/P/B00005JNOG.01._SCTHUMBZZZ_.jpg</URL>
#        <Height Units="pixels">75</Height>
#        <Width Units="pixels">53</Width>
#      </SmallImage>
#      <MediumImage>
#        <URL>http://images.amazon.com/images/P/B00005JNOG.01._SCMZZZZZZZ_.jpg</URL>
#        <Height Units="pixels">160</Height>
#        <Width Units="pixels">113</Width>
#      </MediumImage>
#      <LargeImage>
#        <URL>http://images.amazon.com/images/P/B00005JNOG.01._SCLZZZZZZZ_.jpg</URL>
#         <Height Units="pixels">500</Height>
#        <Width Units="pixels">353</Width>
#      </LargeImage>
#      <ImageSets>
#        <ImageSet Category="primary">
#          <SmallImage>
#            <URL>http://images.amazon.com/images/P/B00005JNOG.01._SCTHUMBZZZ_.jpg</URL>
#            <Height Units="pixels">75</Height>
#            <Width Units="pixels">53</Width>
#          </SmallImage>
#          <MediumImage>
#            <URL>http://images.amazon.com/images/P/B00005JNOG.01._SCMZZZZZZZ_.jpg</URL>
#            <Height Units="pixels">160</Height>
#            <Width Units="pixels">113</Width>
#          </MediumImage>
#          <LargeImage>
#            <URL>http://images.amazon.com/images/P/B00005JNOG.01._SCLZZZZZZZ_.jpg</URL>
#            <Height Units="pixels">500</Height>
#            <Width Units="pixels">353</Width>
#          </LargeImage>
#        </ImageSet>
#      </ImageSets>
#      <OfferSummary>
#        <LowestNewPrice>
#          <Amount>1249</Amount>
#          <CurrencyCode>USD</CurrencyCode>
#          <FormattedPrice>$12.49</FormattedPrice>
#        </LowestNewPrice>
#        <LowestUsedPrice>
#          <Amount>1043</Amount>
#          <CurrencyCode>USD</CurrencyCode>
#          <FormattedPrice>$10.43</FormattedPrice>
#        </LowestUsedPrice>
#        <LowestCollectiblePrice>
#          <Amount>1465</Amount>
#          <CurrencyCode>USD</CurrencyCode>
#          <FormattedPrice>$14.65</FormattedPrice>
#        </LowestCollectiblePrice>
#        <TotalNew>60</TotalNew>
#        <TotalUsed>58</TotalUsed>
#        <TotalCollectible>10</TotalCollectible>
#        <TotalRefurbished>0</TotalRefurbished>
#      </OfferSummary>
#      <Offers>
#        <TotalOffers>1</TotalOffers>
#        <TotalOfferPages>1</TotalOfferPages>
#        <Offer>
#          <Merchant>
#            <MerchantId>ATVPDKIKX0DER</MerchantId>
#            <GlancePage>http://www.amazon.com/gp/help/...</GlancePage>
#          </Merchant>
#          <OfferAttributes>
#            <Condition>New</Condition>
#          </OfferAttributes>
#          <OfferListing>
#            <OfferListingId>...</OfferListingId>
#            <Price>
#              <Amount>1798</Amount>
#              <CurrencyCode>USD</CurrencyCode>
#              <FormattedPrice>$17.98</FormattedPrice>
#            </Price>
#            <Availability>Usually ships in 24 hours</Availability>
#            <IsEligibleForSuperSaverShipping>1</IsEligibleForSuperSaverShipping>
#          </OfferListing>
#        </Offer>
#      </Offers>
#      <ItemAttributes>
#        <Director>Greg Yaitanes</Director>
#        <Director>Tucker Gates</Director>
#        <Director>Michael Zinberg</Director>
#        <Director>J.J. Abrams</Director>
#        <Director>Marita Grabiak</Director>
#        <Director>Alan Taylor</Director>
#        <ProductGroup>DVD</ProductGroup>
#        <Title>Lost - The Complete First Season</Title>
#      </ItemAttributes>
#    </Item>
#  </Items>
#</ItemLookupResponse>


<Item>
  <ASIN>B00005JLXH</ASIN>
</Item>
