# Media Manager Movable Type Plugin
#
# $Id: $
#
# Copyright (C) 2005 Byrne Reese
#

package MediaManager::BQTUpgrade;
use strict;

use vars qw( @ISA );

use MT::App;
use MT::App::CMS;
@ISA = qw(MT::App);

use MT::PluginData;
use MT::Util qw( format_ts offset_time_list );
use MT::ConfigMgr;
use MediaManager::Entry;
use MediaManager::Review;
use MediaManager::Util;
use XML::Simple;

sub init {
    my $app = shift;
    my %param = @_;
    $app->SUPER::init(%param) or return;

    MediaManager::Util::debug("Initializing Media Manager");
    $app->add_methods(
            'default' => \&default,
            'upgrade' => \&upgrade,
    );

    my $p = $MT::Plugin::MTAmazon3::plugin;
    if (!defined($p)) {
	die "It appears that you do not have MTAmazon installed. That is a prerequisite for Media Manager. Please consult the installation instructions.";
    }

    require MTAmazon3::Util;
    $app->{mmanager_cfg}   = MTAmazon3::Util::readconfig($app->{query}->param('blog_id'));
    $app->{default_mode}   = 'default';
    $app->{user_class}     = 'MT::Author';
    $app->{requires_login} = 1;
    MediaManager::Util::debug("Finished initializing Media Manager.");
    $app;
}

sub default {
    my $app = shift;
    my $tmpl = $app->init_tmpl('upgrade.tmpl');
    $tmpl->param('blog_id'  => $app->{query}->param('blog_id'));

    require MediaManager::Entry;
    my $total = MediaManager::Entry->count( {} );
    $tmpl->param(data_exists => $total > 0);

    $app->add_breadcrumb("Main Menu",$app->{mtscript_url});
    $app->add_breadcrumb("Book Queue Too Upgrade");

    $app->{breadcrumbs}[-1]{is_last} = 1;
    $tmpl->param(breadcrumbs => $app->{breadcrumbs});
    $tmpl->param(plugin_version    => $MT::Plugin::MediaManager::VERSION);

    $app->l10n_filter($tmpl->output);
}

sub dont_ask_again {
    my $data = MT::PluginData->new;
    $data->plugin('mediamanager');
    $data->key('upgraded');
    $data->data(1);
    $data->save or die "Unable to save setup confirmation", $data->errstr;
}

sub upgrade {
    my $app = shift;
    if ($app->{query}->param('upgrade') eq "No, Skip this Step") {
	$app->dont_ask_again;
	$app->redirect($app->{cfg}->CGIPath.'plugins/MediaManager/install.cgi?__mode=complete&amp;blog_id='.$app->{query}->param('blog_id'));
	return;
    }
    MediaManager::Util::debug("Upgrading...", "> ");
    MediaManager::Util::debug("Removing previous Media Manager data", "  > ");
    MediaManager::Entry->remove_all;
    MediaManager::Review->remove_all;

    MediaManager::Util::debug("Migrating BookQueue data...", "  > ");
    my @old_entries = MT::PluginData->load({ plugin    => 'BookQueue' },
					   { sort      => 'key',
					     direction => 'descend'   });
    my $i = 0;
    foreach my $old_entry (@old_entries) {
	$i++;
	my $new_entry   = MediaManager::Entry->new;

	require MTAmazon3::Plugin;
	my $content_tree;
	eval {
	    $content_tree = MTAmazon3::Plugin::ItemLookupHelper(
                $app->{mmanager_cfg},{
		    ItemId => $old_entry->data->{isbn},
		    ResponseGroup => 'Small,Images,OfferSummary',
            });
        };
	if ($@) {
	    return $app->error($@);
	}
	my $item = $content_tree->{'Items'}->{'Item'};

	MediaManager::Util::debug("Finished on for '$item->{ProductName}' was: ".$old_entry->data->{finished_on},"  > ");
	my $finished_on;
	if ($old_entry->data->{finished_on} =~
	    m!^(\d{2})-(\d{2})-(\d{4})\s+(\d{2}):(\d{2})(?::(\d{2}))?$!) {
	    $finished_on = $3.$1.$2."000000";
	} elsif ($old_entry->data->{finished_on} =~
	    m!^(\d{4})-(\d{2})-(\d{2})\s+(\d{2}):(\d{2})(?::(\d{2}))?$!) {
	    $finished_on = $1.$2.$3."000000";

	} elsif ($old_entry->data->{finished_on} =~
	    m!^(\d{2})-(\d{4})-(\d{2})\s+(\d{2}):(\d{2})(?::(\d{2}))?$!) {
	    $finished_on = $2.$1.$3."000000";

	} elsif ($old_entry->data->{finished_on} eq "") {
	    $finished_on = undef;

	} else {
	    $finished_on = format_ts("%Y%m%d000000",
				     $old_entry->data->{finished_on},
				     $app->blog);
	}
	MediaManager::Util::debug("Finished on for '$item->{ProductName}' is now: $finished_on","  > ");

	if ($finished_on =~ m!^(\d{2})-(\d{2})-(\d{4})$!) {
	    $finished_on = $3.$1.$2."000000";
	}

	my $catalog = $old_entry->data->{catalog};
	$catalog ||= "Books";

	$new_entry->finished_on($finished_on) if $finished_on;
	$new_entry->created_on($old_entry->data->{created_on});
	$new_entry->blog_id($old_entry->data->{blog_id});
	$new_entry->catalog($item->{ItemAttributes}->{ProductGroup});
	$new_entry->isbn($old_entry->data->{isbn});
	$new_entry->status($old_entry->data->{status});
	$new_entry->title($item->{ItemAttributes}->{Title});
	$new_entry->entry_id($old_entry->data->{entry_id});

	MediaManager::Util::debug("Adding '".$item->{ProductName}."' to datastore.","  > ");
	if ($old_entry->data->{isbn}) {
	    $new_entry->save
		or die $new_entry->errstr;
	}
    }

    $app->dont_ask_again();
    MediaManager::Util::debug("Upgrade complete.", "> ");
    $app->redirect($app->{cfg}->CGIPath.'plugins/MediaManager/mmanager.cgi?__mode=view&amp;blog_id='.$app->{query}->param('blog_id'));
}

sub init_tmpl {
    my $app = shift;
    MediaManager::Util::debug("Initializing template file.","  >");
    MediaManager::Util::debug("Calling MT::App::load_tmpl(".join(", ",@_).")","    >");
    my $tmpl = $app->load_tmpl(@_);
    if (!$tmpl) {
	my $err = $app->translate("Loading template '[_1]' ".
				  "failed: [_2]",
				  $_[0], $@);
	MediaManager::Util::debug($err,"    >");
	return $app->error($err);
    } else {
	MediaManager::Util::debug("Template file successfully loaded.","    >");
    }

    $tmpl->param(plugin_name       => "Media Manager");
    $tmpl->param(plugin_version    => $MT::Plugin::MediaManager::VERSION);
    $tmpl->param(plugin_author     => "Byrne Reese");
    $tmpl->param(author_name       => $app->{author}->name);
    $tmpl->param(nav_mediamanager  => 1);

    MediaManager::Util::debug("Finished initializing template file.","  >");
    return $tmpl;
}

1;
