# Media Manager Movable Type Plugin
#
# $Id: $
#
# Copyright (C) 2005 Byrne Reese
#

package MediaManager::Install;
use strict;

use vars qw( $DEBUG @ISA );

use MT::App;
use MT::App::CMS;
use MT::ConfigMgr;
use MT::Template;
use MT::Blog;
@ISA = qw(MT::App);

use MediaManager::Entry;
use MediaManager::Util;

sub init {
    my $app = shift;
    my %param = @_;
    $app->SUPER::init(%param) or return;

    $app->add_methods(
            'default' => \&default,
            'install' => \&install,
            'complete' => \&complete,
    );

    $app->{default_mode}   = 'default';
    $app->{user_class}     = 'MT::Author';
    $app->{requires_login} = 1;
    $app;
}

sub repair_class {
    require MT::App;
    my $app = MT::App->instance;

    my ($class) = @_;
    require MTAmazon3::Util;
    require MT::Upgrade;
    my $driver = MT::Object->driver;
    my $diff   = MT::Upgrade->class_diff($class);
    if ($diff) {
        my @stmt;
        if ($diff->{fix}) {
            @stmt = $driver->fix_class($class);
        } else {
            if ($diff->{add}) {
                push @stmt, $driver->add_column($class, $_->{name})
                    foreach @{$diff->{add}};
            }
            if ($diff->{alter}) {
                push @stmt, $driver->alter_column($class, $_->{name})
                    foreach @{$diff->{alter}};
            }
            if ($diff->{drop}) {
                push @stmt, $driver->drop_column($class, $_->{name})
                    foreach @{$diff->{drop}};
            }
        }
        if (@stmt) {
            $driver->sql(\@stmt) or return $app->error($driver->errstr);
        }
    }
}

sub default {
    my $app = shift;
    my $tmpl = $app->init_tmpl('install.tmpl');
    $tmpl->param('blog_id'  => $app->{query}->param('blog_id'));

    require MT::Object;
    my $driver = MT::Object->driver;
    if ($driver && $driver->table_exists('MediaManager::Entry')) {
	$tmpl->param('installed' => 1);
	my $err = $app->translate("Media Manager has already been installed. In order to protect the data that may already have been entered by users, this action is being prevented. To re-run the installation process you will need to drop the database tables 'mt_mediamanager' and 'mt_mediamanager_reviews' from your database.");
	return $app->error($err);
    }

    $app->add_breadcrumb("Main Menu",$app->{mtscript_url});
    $app->add_breadcrumb("Media Manager Install");

    $app->{breadcrumbs}[-1]{is_last} = 1;
    $tmpl->param(breadcrumbs => $app->{breadcrumbs});
    $tmpl->param(plugin_version    => $MT::Plugin::MediaManager::VERSION);

    $app->l10n_filter($tmpl->output);
}

sub complete {
    my $app = shift;
    my $tmpl = $app->init_tmpl('install_complete.tmpl');
    $tmpl->param('blog_id'  => $app->{query}->param('blog_id'));

    $app->add_breadcrumb("Main Menu",$app->{mtscript_url});
    $app->add_breadcrumb("Media Manager Install");

    $app->{breadcrumbs}[-1]{is_last} = 1;
    $tmpl->param(breadcrumbs => $app->{breadcrumbs});
    $tmpl->param(plugin_version    => $MT::Plugin::MediaManager::VERSION);

    $app->l10n_filter($tmpl->output);
}

sub install {
    my $app = shift;
    MediaManager::Util::debug("Installing Database...", "> ");
    
    require MT::Object;
    my $driver = MT::Object->driver;
    if (!$driver || !$driver->{dbh}) {
	my $err = $app->translate("Media Manager could not load your database driver. It is possible that you are using an unsupported database. It is suggested you contact Media Manager support and let them know about this error, and what database you are currently using.");
	return $app->error($err);
    }
    if ($driver && $driver->table_exists('MediaManager::Entry')) {
	my $err = $app->translate("Media Manager has already been installed. In order to protect the data that may already have been entered by users, this action is being prevented. To re-run the installation process you will need to drop the database tables 'mt_mediamanager' and 'mt_mediamanager_reviews' from your database.");
	return $app->error($err);
    }

    my $cfg = MT::ConfigMgr->instance;
    my $path = $ENV{SCRIPT_FILENAME};
    substr($path, (-1 * length('install.cgi'))) = "";
    
    $driver->{dbh}->do('drop table mt_mediamanager');
    $driver->{dbh}->do('drop table mt_mediamanager_reviews');
    repair_class('MediaManager::Entry');
    repair_class('MediaManager::Review');
    MediaManager::Util::debug("Installation complete.", "  > ");

    if ($driver && !$driver->table_exists('MediaManager::Entry')) {
	my $err = $app->translate("Media Manager could not install the requisite database tables. This is most likely due to your use of an unsupported or unknown database. This technically should not happen. If you encounter this error, please log a help ticket at the Media Manager homepage.");
	return $app->error($err);
    }

    require MT::PluginData;
    MediaManager::Util::debug("Checking to see if you need an upgrade","  > ");

    my $do_upgrade = MT::PluginData->load({ plugin => 'BookQueue' });
    if ($do_upgrade) {
	# redirect to bqtupgrade
	$app->redirect($app->{cfg}->CGIPath.'plugins/MediaManager/bqtupgrade.cgi?blog_id='.$app->{query}->param('blog_id'));
    } else {
	$app->redirect($app->{cfg}->CGIPath.'plugins/MediaManager/install.cgi?__mode=complete&amp;blog_id='.$app->{query}->param('blog_id'));
    }

    my $iter = MT::Blog->load_iter( { } );
    while (my $blog = $iter->()) {
	my $tmpl;
	$tmpl = MT::Template->new;
	$tmpl->blog_id($blog->id);
	$tmpl->type('custom');
	$tmpl->name('Sidebar: Reading List');
	$tmpl->text(<<END_TMPL);
                     <div class="module-archives module">
                        <h2 class="module-header"><MT_TRANS phrase="Recent Posts"></h2>
                        <div class="module-content">
                           <ul class="module-list">
                           <MTEntries lastn="10">
                              <li class="module-list-item"><a href="<\$MTEntryPermalink\$>"><\$MTEntryTitle\$></a></li>
                           </MTEntries>
                           </ul>
                        </div>
                     </div>

<div class="module-mmanager module">
  <h2 class="module-header"><MT_TRANS phrase="What I Am Reading"></h2>
  <div class="module-content">
<MTMediaManagerItems status="reading" sort_order="descend" sort_by="modified_on" catalog="Books">
<MTAmazonItemLookup ItemId="[MTBookQueueEntryISBN]">
    <div class="amazon-item">
      <a href="<\$MTAmazonDetailPageURL\$>">
        <\$MTAmazonImageTag size="small"\$>
      </a>
      <p><a href="<\$MTAmazonDetailPageURL\$>"><\$MTAmazonTitle\$></a><br />by <\$MTAmazonItemAttributes name="Author"\$></p>
    </div>
</MTAmazonItemLookup>
</MTMediaManagerItems>
    <p><a href="http://www.majordojo.com/projects/MediaManager/">Powered by Media Manager</a></p>
  </div>  
</div>
END_TMPL
        $tmpl->save;

	$tmpl = MT::Template->new;
	$tmpl->blog_id($blog->id);
	$tmpl->type('custom');
	$tmpl->name('Sidebar: Listening To');
	$tmpl->text(<<END_TMPL);
<div class="module-mmanager module">
  <h2 class="module-header"><MT_TRANS phrase="What I Am Listening To"></h2>
  <div class="module-content">
<MTMediaManagerItems status="reading" sort_order="descend" sort_by="modified_on" catalog="Music">
<MTAmazonItemLookup ItemId="[MTBookQueueEntryISBN]">
    <div class="amazon-item">
      <a href="<\$MTAmazonDetailPageURL\$>">
        <\$MTAmazonImageTag size="small"\$>
      </a>
      <p><a href="<\$MTAmazonDetailPageURL\$>"><\$MTAmazonTitle\$></a><br />by <\$MTAmazonItemAttributes name="Artist"\$></p>
    </div>
</MTAmazonItemLookup>
</MTMediaManagerItems>
    <p><a href="http://www.majordojo.com/projects/MediaManager/">Powered by Media Manager</a></p>
  </div>  
</div>
END_TMPL
        $tmpl->save;

	$tmpl = MT::Template->new;
	$tmpl->blog_id($blog->id);
	$tmpl->type('custom');
	$tmpl->name('Sidebar: Last 5 Media Manager Items');
	$tmpl->text(<<END_TMPL);
<div class="module-mmanager module">
  <h2 class="module-header"><MT_TRANS phrase="Recently Added Media"></h2>
  <div class="module-content">
<MTMediaManagerItems sort_order="descend" sort_by="created_on">
<MTAmazonItemLookup ItemId="[MTBookQueueEntryISBN]">
    <div class="amazon-item">
      <a href="<\$MTAmazonDetailPageURL\$>">
        <\$MTAmazonImageTag size="small"\$>
      </a>
      <p><a href="<\$MTAmazonDetailPageURL\$>"><\$MTAmazonTitle\$></a></p>
    </div>
</MTAmazonItemLookup>
</MTMediaManagerItems>
    <p><a href="http://www.majordojo.com/projects/MediaManager/">Powered by Media Manager</a></p>
  </div>
</div>
END_TMPL
        $tmpl->save;
    }
}
    

sub check {
    my $app = shift;
    MediaManager::Util::debug("Installing Database...", "> ");

    # evaluate prerequisites
    my @REQ = (
	       [ 'HTML::Template', 2, 1, 
		 'HTML::Template is required for all Media Manager application '.
		 'functionality.' ],
	       [ 'XML::SAX', 0, 1, 
		 'XML::SAX is required.' ],
	       [ 'XML::NamespaceSupport', 0, 1, 
		 'XML::NamespaceSupport is required.' ],
	       [ 'XML::SAX::PurePerl', 0, 1, 
		 'XML::SAX::PurePerl is required.' ],
	       [ 'XML::Simple', 0, 1, 
		 'XML::Simple is required.' ],
	       [ 'LWP::Simple', 0, 1, 
		 'LWP::Simple is required.' ],
	       );

    MediaManager::Util::debug("Testing extenal prerequisites","  >");
    for my $ref (@REQ) {
        my($mod, $ver, $req, $desc) = @$ref;
        MediaManager::Util::debug("Testing for $mod","    >");
        eval("use $mod" . ($ver ? " $ver;" : ";"));
        if ($@) {
            my $msg = $ver ?
		"Either your server does not have $mod installed, " .
		"the version that is installed is too old, or $mod " . 
		"requires another module that is not installed. ":
		"Your server does not have $mod " . 
		"installed, or $mod requires another module that " . 
		"is not installed. ";
            $msg .= $desc .
		" Please consult the installation instructions for " .
		"help in installing $mod.\n\n";
	    return $app->error($msg);
	}
    }

}

sub init_tmpl {
    my $app = shift;
    MediaManager::Util::debug("Initializing template file.","  >");
    MediaManager::Util::debug("Calling MT::App::load_tmpl(".join(", ",@_).")","    >");
    my $tmpl = $app->load_tmpl(@_);
    if (!$tmpl) {
	my $err = $app->translate("Loading template '[_1]' ".
				  "failed: [_2]",
				  $_[0], $@);
	MediaManager::Util::debug($err,"    >");
	return $app->error($err);
    } else {
	MediaManager::Util::debug("Template file successfully loaded.","    >");
    }

    $tmpl->param(plugin_name       => "Media Manager");
    $tmpl->param(plugin_version    => $MediaManager::VERSION);
    $tmpl->param(plugin_author     => "Byrne Reese");
    $tmpl->param(page_titles       => [ reverse @{ $app->{breadcrumbs} } ]);

    MediaManager::Util::debug("MT Script URL: ".$tmpl->param('mtscript_url'),"    >");

    MediaManager::Util::debug("Finished initializing template file.","  >");
    return $tmpl;
}

1;
